<!DOCTYPE html>
<html class="no-js" lang="de">
<head>
  <meta charset="utf-8">
  <title>AusweisApp2 Release Notes</title>
  <style>
    .important {
      color: red;
    }
  </style>
</head>
<body class="wy-body-for-nav" role="document">
  <div class="wy-grid-for-nav">

    <section data-toggle="wy-nav-shift" class="wy-nav-content-wrap">
      <div class="wy-nav-content">
        <div>
          <div role="main" class="document" itemscope="itemscope" itemtype="http://schema.org/Article">
           <div itemprop="articleBody">
            {% block body %}{% endblock %}
           </div>
          </div>
        </div>
      </div>
    </section>

  </div>
</body>
</html>
