/*!
 * \copyright Copyright (c) 2017-2020 Governikus GmbH & Co. KG, Germany
 */

#include "DataChannel.h"


using namespace governikus;


DataChannel::DataChannel()
{
}


DataChannel::~DataChannel()
{
}
