/*!
 * \copyright Copyright (c) 2018-2020 Governikus GmbH & Co. KG, Germany
 */

#pragma once


namespace governikus
{

class PlatformTools
{
	public:
		static void hideFromTaskbar();
		static void restoreToTaskbar();
};

} // namespace governikus
