// http://msdn.microsoft.com/en-us/library/aa381058(VS.85).aspx
#include <windows.h>
#include "init/config.h"

#define EXECUTABLE   PRODUCT ".exe"

IDR_MAINFRAME ICON "../resources/images/desktop/npa.ico"
IDI_ICON1 ICON DISCARDABLE "../resources/images/desktop/npa.ico"

1 VERSIONINFO
FILEVERSION VERSION_MAJOR,VERSION_MINOR,VERSION_PATCH,VERSION_TWEAK
PRODUCTVERSION VERSION_MAJOR,VERSION_MINOR,VERSION_PATCH,VERSION_TWEAK
FILEOS VOS_NT_WINDOWS32
FILETYPE VFT_APP

BEGIN
  BLOCK "StringFileInfo"

  BEGIN
    BLOCK "040704B0"            // LANG_GERMAN/SUBLANG_DEFAULT, Unicode CP
    BEGIN
      VALUE "CompanyName",      VENDOR
      VALUE "FileDescription",  PRODUCT
      VALUE "FileVersion",      VERSION
      VALUE "InternalName",     PRODUCT
      VALUE "LegalCopyright",   COPYRIGHT_TEXT
      VALUE "OriginalFilename", EXECUTABLE
      VALUE "ProductName",      PRODUCT
      VALUE "ProductVersion",   VERSION
    END

    BLOCK "040904B0"            // LANG_ENGLISH/SUBLANG_ENGLISH_US, Unicode CP
    BEGIN
      VALUE "CompanyName",      VENDOR
      VALUE "FileDescription",  PRODUCT
      VALUE "FileVersion",      VERSION
      VALUE "InternalName",     PRODUCT
      VALUE "LegalCopyright",   COPYRIGHT_TEXT
      VALUE "OriginalFilename", EXECUTABLE
      VALUE "ProductName",      PRODUCT
      VALUE "ProductVersion",   VERSION
    END
  END

  BLOCK "VarFileInfo"
  BEGIN
    VALUE "Translation", 0x0407, 0x04B0
  END
END
