/*!
 * \brief Generic implementation of the controller for retrieving and presenting diagnosis info.
 *
 * \copyright Copyright (c) 2016-2020 Governikus GmbH & Co. KG, Germany
 */


#include "DiagnosisController.h"


using namespace governikus;


void DiagnosisController::getPcscInfo(QVector<DiagnosisContext::ComponentInfo>& pComponents,
		QVector<DiagnosisContext::ComponentInfo>& pDrivers)
{
	Q_UNUSED(pComponents)
	Q_UNUSED(pDrivers)
}
