/*!
 * \copyright Copyright (c) 2018-2020 Governikus GmbH & Co. KG, Germany
 */

#include "PlatformTools.h"


using namespace governikus;


void PlatformTools::hideFromTaskbar()
{
}


void PlatformTools::restoreToTaskbar()
{
}
