/*!
 * \copyright Copyright (c) 2016-2020 Governikus GmbH & Co. KG, Germany
 */

#include "ActivationContext.h"

using namespace governikus;


ActivationContext::ActivationContext()
	: mSendError()
{
}
