/*!
 * \copyright Copyright (c) 2017-2020 Governikus GmbH & Co. KG, Germany
 */

#include "MockReaderConfiguration.h"

using namespace governikus;


void MockReaderConfiguration::clearReaderConfiguration()
{
	mReaderConfigurationInfos.clear();
}
