/*!
 * \copyright Copyright (c) 2014-2020 Governikus GmbH & Co. KG, Germany
 */

#include "CommandApduBuilder.h"
#include <QLoggingCategory>

using namespace governikus;

Q_DECLARE_LOGGING_CATEGORY(card)

/*
 * The base class CommandApduBuilder
 */
CommandApduBuilder::CommandApduBuilder()
{
}


CommandApduBuilder::~CommandApduBuilder()
{
}
