/*!
 * \copyright Copyright (c) 2018-2020 Governikus GmbH & Co. KG, Germany
 */

#pragma once

#include "AbstractState.h"
#include "context/WorkflowContext.h"
#include "GenericContextContainer.h"

class test_StateEnterPacePassword;

namespace governikus
{

class StateEnterPacePassword
	: public AbstractState
	, public GenericContextContainer<WorkflowContext>
{
	Q_OBJECT
	friend class StateBuilder;
	friend class ::test_StateEnterPacePassword;

	explicit StateEnterPacePassword(const QSharedPointer<WorkflowContext>& pContext);
	virtual void run() override;

	public:
		void onEntry(QEvent* pEvent) override;
};

} // namespace governikus
