/*!
 * \copyright Copyright (c) 2014-2020 Governikus GmbH & Co. KG, Germany
 */

#include "Eac1InputType.h"

using namespace governikus;

Eac1InputType::Eac1InputType()
{
}


Eac1InputType::~Eac1InputType()
{
}
