/*!
 * \copyright Copyright (c) 2020 Governikus GmbH & Co. KG, Germany
 */

#include "ApplicationModel.h"

using namespace governikus;

Q_DECLARE_LOGGING_CATEGORY(qml)

void ApplicationModel::showSettings(const ApplicationModel::Settings& pAction)
{
	qCWarning(qml) << "NOT IMPLEMENTED:" << pAction;
}
