/*!
 * \copyright Copyright (c) 2017-2020 Governikus GmbH & Co. KG, Germany
 */

#include "RemoteServer.h"

#include "AppSettings.h"
#include "Env.h"

using namespace governikus;


RemoteServer::~RemoteServer()
{
}
